@@ -7,7 +7,6 @@ from trytond.backend import TableHandler
 from trytond.pyson import Eval, If, Bool, PYSONEncoder
 from trytond.transaction import Transaction
 from trytond.pool import Pool, PoolMeta
-from trytond.config import CONFIG
 
 __all__ = ['TaxGroup', 'TaxCodeTemplate', 'TaxCode',
     'OpenChartTaxCodeStart', 'OpenChartTaxCode',
@@ -17,12 +16,19 @@ __all__ = ['TaxGroup', 'TaxCodeTemplate', 'TaxCode',
     'AccountTemplateTaxTemplate', 'AccountTemplate2', 'AccountTax', 'Account2']
 __metaclass__ = PoolMeta
 
+KINDS = [
+    ('sale', 'Sale'),
+    ('purchase', 'Purchase'),
+    ('both', 'Both'),
+    ]
+
 
 class TaxGroup(ModelSQL, ModelView):
     'Tax Group'
     __name__ = 'account.tax.group'
     name = fields.Char('Name', size=None, required=True, translate=True)
     code = fields.Char('Code', size=None, required=True)
+    kind = fields.Selection(KINDS, 'Kind', required=True)
 
     @classmethod
     def __register__(cls, module_name):
@@ -33,6 +39,10 @@ class TaxGroup(ModelSQL, ModelView):
         # Migration from 1.4 drop code_uniq constraint
         table.drop_constraint('code_uniq')
 
+    @staticmethod
+    def default_kind():
+        return 'both'
+
 
 class TaxCodeTemplate(ModelSQL, ModelView):
     'Tax Code Template'
@@ -48,16 +58,14 @@ class TaxCodeTemplate(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(TaxCodeTemplate, cls).__setup__()
-        cls._constraints += [
-            ('check_recursion', 'recursive_tax_code'),
-            ]
-        cls._error_messages.update({
-                'recursive_tax_code': \
-                    'You can not create recursive tax codes!',
-                })
         cls._order.insert(0, ('code', 'ASC'))
         cls._order.insert(0, ('account', 'ASC'))
 
+    @classmethod
+    def validate(cls, templates):
+        super(TaxCodeTemplate, cls).validate(templates)
+        cls.check_recursion(templates)
+
     def _get_tax_code_value(self, code=None):
         '''
         Set values for tax code creation.
@@ -85,6 +93,7 @@ class TaxCodeTemplate(ModelSQL, ModelView):
         pool = Pool()
         TaxCode = pool.get('account.tax.code')
         Lang = pool.get('ir.lang')
+        Config = pool.get('ir.configuration')
 
         if template2tax_code is None:
             template2tax_code = {}
@@ -94,9 +103,9 @@ class TaxCodeTemplate(ModelSQL, ModelView):
             vals['company'] = company_id
             vals['parent'] = parent_id
 
-            new_tax_code = TaxCode.create(vals)
+            new_tax_code, = TaxCode.create([vals])
 
-            prev_lang = self._context.get('language') or CONFIG['language']
+            prev_lang = self._context.get('language') or Config.get_language()
             prev_data = {}
             for field_name, field in self._fields.iteritems():
                 if getattr(field, 'translate', False):
@@ -148,15 +157,13 @@ class TaxCode(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(TaxCode, cls).__setup__()
-        cls._constraints += [
-            ('check_recursion', 'recursive_tax_code'),
-            ]
-        cls._error_messages.update({
-                'recursive_tax_code': \
-                    'You can not create recursive tax codes!',
-                })
         cls._order.insert(0, ('code', 'ASC'))
 
+    @classmethod
+    def validate(cls, codes):
+        super(TaxCode, cls).validate(codes)
+        cls.check_recursion(codes)
+
     @staticmethod
     def default_active():
         return True
@@ -238,7 +245,9 @@ class TaxCode(ModelSQL, ModelView):
         tax code id as value, used to convert template id into tax code. The
         dictionary is filled with new tax codes
         '''
-        Lang = Pool().get('ir.lang')
+        pool = Pool()
+        Lang = pool.get('ir.lang')
+        Config = pool.get('ir.configuration')
 
         if template2tax_code is None:
             template2tax_code = {}
@@ -248,7 +257,7 @@ class TaxCode(ModelSQL, ModelView):
             if vals:
                 self.write([self], vals)
 
-            prev_lang = self._context.get('language') or CONFIG['language']
+            prev_lang = self._context.get('language') or Config.get_language()
             prev_data = {}
             for field_name, field in self.template._fields.iteritems():
                 if getattr(field, 'translate', False):
@@ -309,8 +318,7 @@ class OpenChartTaxCode(Wizard):
     def do_open_(self, action):
         if self.start.method == 'fiscalyear':
             action['pyson_context'] = PYSONEncoder().encode({
-                    'fiscalyear': (self.start.fiscalyear.id
-                        if self.start.fiscalyear else None),
+                    'fiscalyear': self.start.fiscalyear.id,
                     })
             if self.start.fiscalyear:
                 action['name'] += ' - %s' % self.start.fiscalyear.rec_name
@@ -447,6 +455,7 @@ class TaxTemplate(ModelSQL, ModelView):
         pool = Pool()
         Tax = pool.get('account.tax')
         Lang = pool.get('ir.lang')
+        Config = pool.get('ir.configuration')
 
         if template2tax is None:
             template2tax = {}
@@ -457,38 +466,38 @@ class TaxTemplate(ModelSQL, ModelView):
             vals['parent'] = parent_id
             if self.invoice_account:
                 vals['invoice_account'] = \
-                        template2account[self.invoice_account.id]
+                    template2account[self.invoice_account.id]
             else:
                 vals['invoice_account'] = None
             if self.credit_note_account:
                 vals['credit_note_account'] = \
-                        template2account[self.credit_note_account.id]
+                    template2account[self.credit_note_account.id]
             else:
                 vals['credit_note_account'] = None
             if self.invoice_base_code:
                 vals['invoice_base_code'] = \
-                        template2tax_code[self.invoice_base_code.id]
+                    template2tax_code[self.invoice_base_code.id]
             else:
                 vals['invoice_base_code'] = None
             if self.invoice_tax_code:
                 vals['invoice_tax_code'] = \
-                        template2tax_code[self.invoice_tax_code.id]
+                    template2tax_code[self.invoice_tax_code.id]
             else:
                 vals['invoice_tax_code'] = None
             if self.credit_note_base_code:
                 vals['credit_note_base_code'] = \
-                        template2tax_code[self.credit_note_base_code.id]
+                    template2tax_code[self.credit_note_base_code.id]
             else:
                 vals['credit_note_base_code'] = None
             if self.credit_note_tax_code:
                 vals['credit_note_tax_code'] = \
-                        template2tax_code[self.credit_note_tax_code.id]
+                    template2tax_code[self.credit_note_tax_code.id]
             else:
                 vals['credit_note_tax_code'] = None
 
-            new_tax = Tax.create(vals)
+            new_tax, = Tax.create([vals])
 
-            prev_lang = self._context.get('language') or CONFIG['language']
+            prev_lang = self._context.get('language') or Config.get_language()
             prev_data = {}
             for field_name, field in self._fields.iteritems():
                 if getattr(field, 'translate', False):
@@ -748,67 +757,57 @@ class Tax(ModelSQL, ModelView):
         value, used to convert template id into tax.  The dictionary is filled
         with new taxes.
         '''
-        Lang = Pool().get('ir.lang')
+        pool = Pool()
+        Lang = pool.get('ir.lang')
+        Config = pool.get('ir.configuration')
 
         if template2tax is None:
             template2tax = {}
 
         if self.template:
             vals = self.template._get_tax_value(tax=self)
-            invoice_account_id = (self.invoice_account.id
-                if self.invoice_account else None)
-            if (self.template.invoice_account and
-                    invoice_account_id != template2account.get(
-                            self.template.invoice_account.id)):
+            if (self.template.invoice_account
+                    and self.invoice_account.id != template2account.get(
+                        self.template.invoice_account.id)):
                 vals['invoice_account'] = template2account.get(
                     self.template.invoice_account.id)
             elif (not self.template.invoice_account
                     and self.invoice_account):
                 vals['invoice_account'] = None
-            credit_note_account_id = (self.credit_note_account.id
-                if self.credit_note_account else None)
-            if (self.template.credit_note_account and
-                    credit_note_account_id != template2account.get(
+            if (self.template.credit_note_account
+                    and self.credit_note_account.id != template2account.get(
                         self.template.credit_note_account.id)):
                 vals['credit_note_account'] = template2account.get(
                     self.template.credit_note_account.id)
             elif (not self.template.credit_note_account
                     and self.credit_note_account):
                 vals['credit_note_account'] = None
-            invoice_base_code_id = (self.invoice_base_code.id
-                if self.invoice_base_code else None)
-            if (self.template.invoice_base_code and
-                    invoice_base_code_id != template2tax_code.get(
-                            self.template.invoice_base_code.id)):
+            if (self.template.invoice_base_code
+                    and self.invoice_base_code.id != template2tax_code.get(
+                        self.template.invoice_base_code.id)):
                 vals['invoice_base_code'] = template2tax_code.get(
                     self.template.invoice_base_code.id)
             elif (not self.template.invoice_base_code
                     and self.invoice_base_code):
                 vals['invoice_base_code'] = None
-            invoice_tax_code_id = (self.invoice_tax_code.id
-                if self.invoice_tax_code else None)
             if (self.template.invoice_tax_code
-                    and invoice_tax_code_id != template2tax_code.get(
+                    and self.invoice_tax_code.id != template2tax_code.get(
                         self.template.invoice_tax_code.id)):
                 vals['invoice_tax_code'] = template2tax_code.get(
                     self.template.invoice_tax_code.id)
             elif (not self.template.invoice_tax_code
                     and self.invoice_tax_code):
                 vals['invoice_tax_code'] = None
-            credit_note_base_code_id = (self.credit_note_base_code.id
-                if self.credit_note_base_code else None)
             if (self.template.credit_note_base_code
-                    and credit_note_base_code_id != template2tax_code.get(
+                    and self.credit_note_base_code.id != template2tax_code.get(
                         self.template.credit_note_base_code.id)):
                 vals['credit_note_base_code'] = template2tax_code.get(
                     self.template.credit_note_base_code.id)
             elif (not self.template.credit_note_base_code
                     and self.credit_note_base_code):
                 vals['credit_note_base_code'] = None
-            credit_note_tax_code_id = (self.credit_note_tax_code.id
-                if self.credit_note_tax_code else None)
             if (self.template.credit_note_tax_code
-                    and credit_note_tax_code_id != template2tax_code.get(
+                    and self.credit_note_tax_code.id != template2tax_code.get(
                         self.template.credit_note_tax_code.id)):
                 vals['credit_note_tax_code'] = template2tax_code.get(
                     self.template.credit_note_tax_code.id)
@@ -819,7 +818,7 @@ class Tax(ModelSQL, ModelView):
             if vals:
                 self.write([self], vals)
 
-            prev_lang = self._context.get('language') or CONFIG['language']
+            prev_lang = self._context.get('language') or Config.get_language()
             prev_data = {}
             for field_name, field in self.template._fields.iteritems():
                 if getattr(field, 'translate', False):
@@ -875,10 +874,15 @@ class TaxRuleTemplate(ModelSQL, ModelView):
     'Tax Rule Template'
     __name__ = 'account.tax.rule.template'
     name = fields.Char('Name', required=True, translate=True)
+    kind = fields.Selection(KINDS, 'Kind', required=True)
     lines = fields.One2Many('account.tax.rule.line.template', 'rule', 'Lines')
     account = fields.Many2One('account.account.template', 'Account Template',
             domain=[('parent', '=', None)], required=True)
 
+    @staticmethod
+    def default_kind():
+        return 'both'
+
     def _get_tax_rule_value(self, rule=None):
         '''
         Set values for tax rule creation.
@@ -886,6 +890,8 @@ class TaxRuleTemplate(ModelSQL, ModelView):
         res = {}
         if not rule or rule.name != self.name:
             res['name'] = self.name
+        if not rule or rule.kind != self.kind:
+            res['kind'] = self.kind
         if not rule or rule.template.id != self.id:
             res['template'] = self.id
         return res
@@ -901,6 +907,7 @@ class TaxRuleTemplate(ModelSQL, ModelView):
         pool = Pool()
         Rule = pool.get('account.tax.rule')
         Lang = pool.get('ir.lang')
+        Config = pool.get('ir.configuration')
 
         if template2rule is None:
             template2rule = {}
@@ -908,9 +915,9 @@ class TaxRuleTemplate(ModelSQL, ModelView):
         if self.id not in template2rule:
             vals = self._get_tax_rule_value()
             vals['company'] = company_id
-            new_rule = Rule.create(vals)
+            new_rule, = Rule.create([vals])
 
-            prev_lang = self._context.get('language') or CONFIG['language']
+            prev_lang = self._context.get('language') or Config.get_language()
             prev_data = {}
             for field_name, field in self._fields.iteritems():
                 if getattr(field, 'translate', False):
@@ -936,6 +943,7 @@ class TaxRule(ModelSQL, ModelView):
     'Tax Rule'
     __name__ = 'account.tax.rule'
     name = fields.Char('Name', required=True, translate=True)
+    kind = fields.Selection(KINDS, 'Kind', required=True)
     company = fields.Many2One('company.company', 'Company', required=True,
         select=True, domain=[
             ('id', If(Eval('context', {}).contains('company'), '=', '!='),
@@ -944,6 +952,10 @@ class TaxRule(ModelSQL, ModelView):
     lines = fields.One2Many('account.tax.rule.line', 'rule', 'Lines')
     template = fields.Many2One('account.tax.rule.template', 'Template')
 
+    @staticmethod
+    def default_kind():
+        return 'both'
+
     def apply(self, tax, pattern):
         '''
         Apply rule on tax
@@ -967,7 +979,9 @@ class TaxRule(ModelSQL, ModelView):
         rule id as value, used to convert template id into tax rule. The
         dictionary is filled with new tax rules.
         '''
-        Lang = Pool().get('ir.lang')
+        pool = Pool()
+        Lang = pool.get('ir.lang')
+        Config = pool.get('ir.configuration')
 
         if template2rule is None:
             template2rule = {}
@@ -977,7 +991,7 @@ class TaxRule(ModelSQL, ModelView):
             if vals:
                 self.write([self], vals)
 
-            prev_lang = self._context.get('language') or CONFIG['language']
+            prev_lang = self._context.get('language') or Config.get_language()
             prev_data = {}
             for field_name, field in self.template._fields.iteritems():
                 if getattr(field, 'translate', False):
@@ -1008,15 +1022,33 @@ class TaxRuleLineTemplate(ModelSQL, ModelView):
     origin_tax = fields.Many2One('account.tax.template', 'Original Tax',
         domain=[
             ('account', '=', Eval('_parent_rule', {}).get('account', 0)),
-            ('group', '=', Eval('group'))
+            ('group', '=', Eval('group')),
+            ['OR',
+                ('group', '=', None),
+                If(Eval('_parent_rule', {}).get('kind', 'both') == 'sale',
+                    ('group.kind', 'in', ['sale', 'both']),
+                    If(Eval('_parent_rule', {}).get('kind', 'both') ==
+                        'purchase',
+                        ('group.kind', 'in', ['purchase', 'both']),
+                        ('group.kind', 'in', ['sale', 'purchase', 'both']))),
+                ],
             ],
-        help='If the original tax template is filled, the rule will be ' \
-            'applied only for this tax template.',
+        help=('If the original tax template is filled, the rule will be '
+            'applied only for this tax template.'),
         depends=['group'])
     tax = fields.Many2One('account.tax.template', 'Substitution Tax',
         domain=[
             ('account', '=', Eval('_parent_rule', {}).get('account', 0)),
             ('group', '=', Eval('group')),
+            ['OR',
+                ('group', '=', None),
+                If(Eval('_parent_rule', {}).get('kind', 'both') == 'sale',
+                    ('group.kind', 'in', ['sale', 'both']),
+                    If(Eval('_parent_rule', {}).get('kind', 'both') ==
+                        'purchase',
+                        ('group.kind', 'in', ['purchase', 'both']),
+                        ('group.kind', 'in', ['sale', 'purchase', 'both']))),
+                ],
             ],
         depends=['group'])
     sequence = fields.Integer('Sequence',
@@ -1046,6 +1078,8 @@ class TaxRuleLineTemplate(ModelSQL, ModelView):
         res = {}
         if not rule_line or rule_line.group != self.group:
             res['group'] = self.group.id if self.group else None
+        if not rule_line or rule_line.origin_tax != self.origin_tax:
+            res['origin_tax'] = self.origin_tax.id if self.origin_tax else None
         if not rule_line or rule_line.sequence != self.sequence:
             res['sequence'] = self.sequence
         if not rule_line or rule_line.template != self:
@@ -1081,7 +1115,7 @@ class TaxRuleLineTemplate(ModelSQL, ModelView):
                 vals['tax'] = template2tax[self.tax.id]
             else:
                 vals['tax'] = None
-            new_rule_line = RuleLine.create(vals)
+            new_rule_line, = RuleLine.create([vals])
             template2rule_line[self.id] = new_rule_line.id
         return template2rule_line[self.id]
 
@@ -1097,14 +1131,32 @@ class TaxRuleLine(ModelSQL, ModelView):
         domain=[
             ('company', '=', Eval('_parent_rule', {}).get('company')),
             ('group', '=', Eval('group')),
+            ['OR',
+                ('group', '=', None),
+                If(Eval('_parent_rule', {}).get('kind', 'both') == 'sale',
+                    ('group.kind', 'in', ['sale', 'both']),
+                    If(Eval('_parent_rule', {}).get('kind', 'both') ==
+                        'purchase',
+                        ('group.kind', 'in', ['purchase', 'both']),
+                        ('group.kind', 'in', ['sale', 'purchase', 'both']))),
+                ],
             ],
-        help='If the original tax is filled, the rule will be applied ' \
-            'only for this tax.',
+        help=('If the original tax is filled, the rule will be applied '
+            'only for this tax.'),
         depends=['group'])
     tax = fields.Many2One('account.tax', 'Substitution Tax',
         domain=[
             ('company', '=', Eval('_parent_rule', {}).get('company')),
             ('group', '=', Eval('group')),
+            ['OR',
+                ('group', '=', None),
+                If(Eval('_parent_rule', {}).get('kind', 'both') == 'sale',
+                    ('group.kind', 'in', ['sale', 'both']),
+                    If(Eval('_parent_rule', {}).get('kind', 'both') ==
+                        'purchase',
+                        ('group.kind', 'in', ['purchase', 'both']),
+                        ('group.kind', 'in', ['sale', 'purchase', 'both']))),
+                ],
             ],
         depends=['group'])
     sequence = fields.Integer('Sequence',
@@ -1177,22 +1229,14 @@ class TaxRuleLine(ModelSQL, ModelView):
                 vals['rule'] = template2rule[self.template.rule.id]
             if self.origin_tax:
                 if self.template.origin_tax:
-                    if (self.origin_tax.id !=
-                            template2tax[self.template.origin_tax.id]):
+                    if self.origin_tax.id != \
+                            template2tax[self.template.origin_tax.id]:
                         vals['origin_tax'] = template2tax[
                             self.template.origin_tax.id]
-                else:
-                    vals['origin_tax'] = None
-            elif self.template.origin_tax:
-                vals['origin_tax'] = template2tax[self.template.origin_tax.id]
             if self.tax:
                 if self.template.tax:
                     if self.tax.id != template2tax[self.template.tax.id]:
                         vals['tax'] = template2tax[self.template.tax.id]
-                else:
-                    vals['tax'] = None
-            elif self.template.tax:
-                vals['tax'] = template2tax[self.template.tax.id]
             if vals:
                 self.write([self], vals)
             template2rule_line[self.template.id] = self.id
@@ -1274,6 +1318,6 @@ class Account2:
                 ('company', '=', Eval('company')),
                 ('parent', '=', None),
             ],
-            help='Default tax for manual encoding of move lines \n' \
-                    'for journal types: "expense" and "revenue"',
+            help=('Default tax for manual encoding of move lines \n'
+                'for journal types: "expense" and "revenue"'),
             depends=['company'])
